﻿using IqrfDpaLib;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace IqrfComm
{
    class Program
    {
        static void Main(string[] args)
        {
            Console.WriteLine("Press ESC to exit application.");

            int triggerCnt = 0;

            IqrfUartComm uartComm = new IqrfUartComm("COM6", 57600, TimeSpan.FromSeconds(2));

            byte[] commBuffer = new byte[128];

            DigitalIoDirectionSettings[] ioDirSettingsPIR = new DigitalIoDirectionSettings[] 
            {
                new DigitalIoDirectionSettings { Port = 0x00, Direction = 0x01, Mask = 0x01 }
            };

            DigitalIoDirectionSettings[] ioDirSettingsRelay = new DigitalIoDirectionSettings[]
            {
                new DigitalIoDirectionSettings { Port = 0x00, Direction = 0x00, Mask = 0x01 },
                new DigitalIoDirectionSettings { Port = 0x02, Direction = 0x00, Mask = 0x04 }
            };

            DigitalIoValueSettings[] ioOuputValSettingsOn = new DigitalIoValueSettings[]
            {
                new DigitalIoValueSettings { Par1 = 0x00, Par2 = 0x01, Par3 = 0x01 },
                new DigitalIoValueSettings { Par1 = 0xFF, Par2 = 0xB8, Par3 = 0x0B },
                new DigitalIoValueSettings { Par1 = 0x00, Par2 = 0x01, Par3 = 0x00 }
            };

            DigitalIoValueSettings[] ioOuputValSettingsOff = new DigitalIoValueSettings[]
            {
                new DigitalIoValueSettings { Par1 = 0x02, Par2 = 0x04, Par3 = 0x04 },
                new DigitalIoValueSettings { Par1 = 0xFF, Par2 = 0xB8, Par3 = 0x0B },
                new DigitalIoValueSettings { Par1 = 0x02, Par2 = 0x04, Par3 = 0x00 }
            };

            DigitalIoValueSettings[] ioInputValSettings = new DigitalIoValueSettings[3];

            try
            {
                Console.WriteLine(DateTime.Now.TimeOfDay + "\tDirection settings PIR");

                var frameLengthPIR = DpaFrame.IoDirectionReq(commBuffer, 1, ioDirSettingsPIR);

                uartComm.Send(commBuffer, frameLengthPIR);
                uartComm.Receive(commBuffer);

                DpaFrame.ParseConfirmationRsp(commBuffer);

                uartComm.Receive(commBuffer);

                DpaFrame.ValidateIoDirectionRsp(commBuffer);

                Console.WriteLine(" \t\t\tOK");
            }
            catch (Exception e)
            {
                Console.WriteLine(" \t\t\tFailed: " + e.Message);
            }

            try
            {
                Console.WriteLine(DateTime.Now.TimeOfDay + "\tDirection settings Relay");

                var frameLengthRelay = DpaFrame.IoDirectionReq(commBuffer, 2, ioDirSettingsRelay);

                uartComm.Send(commBuffer, frameLengthRelay);
                uartComm.Receive(commBuffer);

                DpaFrame.ParseConfirmationRsp(commBuffer);

                uartComm.Receive(commBuffer);

                DpaFrame.ValidateIoDirectionRsp(commBuffer);

                Console.WriteLine(" \t\t\tOK");
            }
            catch (Exception e)
            {
                Console.WriteLine(" \t\t\tFailed: " + e.Message);
            }

            do
            {
                if (triggerCnt++ == 10)
                {

                    triggerCnt = 0;

                    Console.WriteLine(DateTime.Now.TimeOfDay + "\tReading sensor values\n");

                    try
                    {
                        var frameLength = DpaFrame.SensorReq(commBuffer, 100, SensorCmd.Temperature);

                        uartComm.Send(commBuffer, frameLength);
                        uartComm.Receive(commBuffer);

                        DpaFrame.ParseConfirmationRsp(commBuffer);

                        uartComm.Receive(commBuffer);


                        Console.WriteLine(" \t\t\tTemperature value {0}", DpaFrame.ParseSensorRsp(commBuffer, SensorCmd.Temperature));
                    }
                    catch (Exception e)
                    {
                        Console.WriteLine(" \t\t\tFailed: " + e.Message);
                    }

                    try
                    {
                        var frameLength = DpaFrame.SensorReq(commBuffer, 100, SensorCmd.Humidity);

                        uartComm.Send(commBuffer, frameLength);

                        uartComm.Receive(commBuffer);

                        DpaFrame.ParseConfirmationRsp(commBuffer);

                        uartComm.Receive(commBuffer);

                        Console.WriteLine(" \t\t\tHumidity value {0}", DpaFrame.ParseSensorRsp(commBuffer, SensorCmd.Humidity));
                    }
                    catch (Exception e)
                    {
                        Console.WriteLine(" \t\t\tFailed: " + e.Message);
                    }


                    Console.WriteLine(DateTime.Now.TimeOfDay + "\tGetting I/O state\n");
                    
                    try
                    {
                        var frameLength = DpaFrame.IoGetReq(commBuffer, 1); 

                        uartComm.Send(commBuffer, frameLength);

                        uartComm.Receive(commBuffer);

                        DpaFrame.ParseConfirmationRsp(commBuffer);

                        uartComm.Receive(commBuffer);

                        var ioState = DpaFrame.ParseIoGetReq(commBuffer);

                        Console.WriteLine(" \t\tIO State: " + (ioState.PortA & 1));
                    }
                    catch (Exception e)
                    {
                        Console.WriteLine(" \t\tFailed: " + e.Message);
                    }

                }

                if (Console.KeyAvailable)
                {
                    ConsoleKeyInfo key = Console.ReadKey(true);
                    switch (key.Key)
                    {
                        case ConsoleKey.Escape:
                            Environment.Exit(0);
                            break;

                        case ConsoleKey.C:

                            Console.WriteLine(DateTime.Now.TimeOfDay + "\tDirection settings");
                            
                            try
                            {
                                var frameLength = DpaFrame.IoSetReq(commBuffer, 2, ioOuputValSettingsOn);

                                uartComm.Send(commBuffer, frameLength);

                                uartComm.Receive(commBuffer);

                                DpaFrame.ParseConfirmationRsp(commBuffer);

                                uartComm.Receive(commBuffer);

                                DpaFrame.ValidateIoSetRsp(commBuffer);

                            }
                            catch (Exception e)
                            {
                                Console.WriteLine(" \t\tFailed: " + e.Message);
                            }
                            
                            break;

                        case ConsoleKey.O:

                            Console.WriteLine(DateTime.Now.TimeOfDay + "\tDirection settings");

                            try
                            {
                                var frameLength = DpaFrame.IoSetReq(commBuffer, 2, ioOuputValSettingsOff);

                                uartComm.Send(commBuffer, frameLength);

                                uartComm.Receive(commBuffer);

                                DpaFrame.ParseConfirmationRsp(commBuffer);

                                uartComm.Receive(commBuffer);

                                DpaFrame.ValidateIoSetRsp(commBuffer);

                            }
                            catch (Exception e)
                            {
                                Console.WriteLine(" \t\tFailed: " + e.Message);
                            }

                            break;

                        default:
                            break;
                    }
                }

                Thread.Sleep(100);

            } while (true);
        }
    }
}
